DROP TABLE IF EXISTS property;
DROP TABLE IF EXISTS attribute_visibility;
DROP TABLE IF EXISTS attribute;
DROP TABLE IF EXISTS label_item;
DROP TABLE IF EXISTS edge_item;
DROP TABLE IF EXISTS node_item;
DROP TABLE IF EXISTS meta_attributes_to_graphic;
DROP TABLE IF EXISTS graphics_to_view;
DROP TABLE IF EXISTS mutant_connection;
DROP TABLE IF EXISTS mutant_object;
DROP TABLE IF EXISTS meta_attribute;
DROP TABLE IF EXISTS view_model;
DROP TABLE IF EXISTS model;
DROP TABLE IF EXISTS project;
DROP TABLE IF EXISTS graphic;
DROP TABLE IF EXISTS view;
DROP TABLE IF EXISTS meta_model;

CREATE TABLE meta_model (
	id VARCHAR(255) NOT NULL,
	name VARCHAR(64) NOT NULL,
	author VARCHAR(64),
	version INTEGER DEFAULT 1 NOT NULL,
	date TIMESTAMP(0) DEFAULT CURRENT_TIMESTAMP NOT NULL,
	PRIMARY KEY(id)
);

CREATE TABLE view (
	id VARCHAR(255) NOT NULL,
	meta_model VARCHAR(255) NOT NULL,
	name VARCHAR(64) NOT NULL,
	author VARCHAR(64),
	version INTEGER DEFAULT 1 NOT NULL,
	date TIMESTAMP(0) DEFAULT CURRENT_TIMESTAMP NOT NULL,
	global BOOLEAN DEFAULT FALSE NOT NULL,
	PRIMARY KEY(id),
	CONSTRAINT FK_VIEW_META_MODEL FOREIGN KEY(meta_model) REFERENCES meta_model(id) ON DELETE CASCADE
);

-- The column "type":
--		"n" stands for node,
--		"e" stands for edge.
CREATE TABLE graphic (
	id VARCHAR(255) NOT NULL,
	meta_model VARCHAR(255) NOT NULL,
	shape VARCHAR(255) NOT NULL,
	path VARCHAR(255) NOT NULL,
	type CHAR(1) DEFAULT 'n' NOT NULL,
	CONSTRAINT FK_GRAPHIC_META_MODEL FOREIGN KEY(meta_model) REFERENCES meta_model(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

CREATE TABLE project (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	meta_model VARCHAR(255),
	name VARCHAR(255) NOT NULL,
	localname VARCHAR(255),
	publicproject VARCHAR(255),
	state VARCHAR(255),
	version VARCHAR(255),
	location VARCHAR(255),
	area VARCHAR(255),
	attentionGroup VARCHAR(255),
	contentLanguage VARCHAR(255),
	author VARCHAR(255),
	mail VARCHAR(255),
	url VARCHAR(255),
	validFor VARCHAR(255),
	validUntil DATE DEFAULT CURRENT_DATE,
	date TIMESTAMP(0) DEFAULT CURRENT_TIMESTAMP NOT NULL,
	description LONGVARCHAR,
	CONSTRAINT FK_PROJECT_META_MODEL FOREIGN KEY(meta_model) REFERENCES meta_model(id),
	PRIMARY KEY(id)
);

CREATE TABLE model (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	project INTEGER NOT NULL,
	view VARCHAR(255),
	name VARCHAR(255) NOT NULL,
	localname VARCHAR(255),
	publicmodel VARCHAR(255),
	state VARCHAR(255),
	version VARCHAR(255),
	location VARCHAR(255),
	area VARCHAR(255),
	attentionGroup VARCHAR(255),
	contentLanguage VARCHAR(255),
	author VARCHAR(255),
	mail VARCHAR(255),
	url VARCHAR(255),
	validFor VARCHAR(255),
	validUntil DATE DEFAULT CURRENT_DATE,
	date TIMESTAMP(0) DEFAULT CURRENT_TIMESTAMP NOT NULL,
	description LONGVARCHAR,
	PRIMARY KEY(id),
	CONSTRAINT FK_MODEL_PROJECT FOREIGN KEY(project) REFERENCES project(id) ON DELETE CASCADE,
	CONSTRAINT FK_MODEL_VIEW FOREIGN KEY(view) REFERENCES view(id)
);

CREATE TABLE meta_attribute (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	at_id VARCHAR(64) UNIQUE NOT NULL,
	name VARCHAR(64) UNIQUE NOT NULL,
	data_type VARCHAR(64) NOT NULL,
	PRIMARY KEY(id)
);

CREATE TABLE mutant_object (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	mo_id VARCHAR(64) UNIQUE NOT NULL,
	default_shape VARCHAR(255),
	CONSTRAINT FK_MUTANT_OBJECT_DEFAULT_GRAPHIC FOREIGN KEY(default_shape) REFERENCES graphic(id),
	PRIMARY KEY(id)
);

CREATE TABLE mutant_connection (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	mc_id VARCHAR(64) UNIQUE NOT NULL,
	default_shape VARCHAR(255),
	source INTEGER,
	target INTEGER,
	source_connection INTEGER,
	target_connection INTEGER,
	CONSTRAINT FK_MUTANT_CONNECTION_DEFAULT_GRAPHIC FOREIGN KEY(default_shape) REFERENCES graphic(id),
	CONSTRAINT FK_MUTANT_CONNECTION_SOURCE_MUTANT_OBJECT FOREIGN KEY(source) REFERENCES mutant_object(id) ON DELETE CASCADE,
	CONSTRAINT FK_MUTANT_CONNECTION_TARGET_MUTANT_OBJECT FOREIGN KEY(target) REFERENCES mutant_object(id) ON DELETE CASCADE,
	CONSTRAINT FK_MUTANT_CONNECTION_SOURCE_MUTANT_CONNECTION FOREIGN KEY(source_connection) REFERENCES mutant_connection(id) ON DELETE CASCADE,
	CONSTRAINT FK_MUTANT_CONNECTION_TARGET_MUTANT_CONNECTION FOREIGN KEY(target_connection) REFERENCES mutant_connection(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

-- This table assigns the meta model graphics to the single views.
CREATE TABLE graphics_to_view (
	id VARCHAR(255) NOT NULL,
	view VARCHAR(255) NOT NULL,
	graphic VARCHAR(255) NOT NULL,
	labeled BOOLEAN DEFAULT TRUE NOT NULL,
	h_align CHAR(1) DEFAULT 2,
	v_align CHAR(1) DEFAULT 2,
	UNIQUE(view, graphic),
	CONSTRAINT FK_GRAPHICS_TO_VIEW_VIEW FOREIGN KEY(view) REFERENCES view(id) ON DELETE CASCADE,
	CONSTRAINT FK_GRAPHICS_TO_VIEW_GRAPHIC FOREIGN KEY(graphic) REFERENCES graphic(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

-- This table assigns the attributes being valid for all nodes and edges of one graphic to said graphic (filter).
CREATE TABLE meta_attributes_to_graphic (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	meta_attribute INTEGER NOT NULL,
	graphics_to_view VARCHAR(255) NOT NULL,
	UNIQUE(meta_attribute, graphics_to_view),
	CONSTRAINT FK_META_ATTRIBUTES_TO_GRAPHIC_META_ATTRIBUTE FOREIGN KEY(meta_attribute) REFERENCES meta_attribute(id) ON DELETE CASCADE,
	CONSTRAINT FK_META_ATTRIBUTES_TO_GRAPHIC_GRAPHICS_TO_VIEW FOREIGN KEY(graphics_to_view) REFERENCES graphics_to_view(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

-- Attributes referenced by mutant objects or connections.
CREATE TABLE attribute (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	meta_attribute INTEGER NOT NULL,
	mutant_object INTEGER,
	mutant_connection INTEGER,
	meta_attributes_to_graphic INTEGER,
	value LONGVARCHAR NOT NULL,
	CONSTRAINT FK_ATTRIBUTE_META_ATTRIBUTE FOREIGN KEY(meta_attribute) REFERENCES meta_attribute(id) ON DELETE CASCADE,
	CONSTRAINT FK_ATTRIBUTE_MUTANT_OBJECT FOREIGN KEY(mutant_object) REFERENCES mutant_object(id) ON DELETE CASCADE,
	CONSTRAINT FK_ATTRIBUTE_MUTANT_CONNECTION FOREIGN KEY(mutant_connection) REFERENCES mutant_connection(id) ON DELETE CASCADE,
	CONSTRAINT FK_ATTRIBUTE_META_ATTRIBUTES_TO_GRAPHIC FOREIGN KEY(meta_attributes_to_graphic) REFERENCES meta_attributes_to_graphic(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

-- This table contains the nodes of the graph.
CREATE TABLE node_item (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	model INTEGER NOT NULL,
	mutant_object INTEGER NOT NULL,
	shape VARCHAR(255),
	parent INTEGER,
	x DOUBLE,
	y DOUBLE,
	z INTEGER DEFAULT 268435456,
	width DOUBLE,
	height DOUBLE,
	size DOUBLE DEFAULT 1.0,
	fill_color INTEGER,
	stroke_color INTEGER,
	stroke_width DOUBLE,
	labeled BOOLEAN DEFAULT TRUE,
	visible BOOLEAN DEFAULT TRUE,
	data_group VARCHAR(64),
	CONSTRAINT FK_NODE_MODEL FOREIGN KEY (model) REFERENCES model(id) ON DELETE CASCADE,
	CONSTRAINT FK_NODE_MUTANT_OBJECT FOREIGN KEY (mutant_object) REFERENCES mutant_object(id) ON DELETE CASCADE,
	CONSTRAINT FK_NODE_SHAPE FOREIGN KEY (shape) REFERENCES graphic(id),
	CONSTRAINT FK_NODE_PARENT FOREIGN KEY(parent) REFERENCES node_item(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

-- This table contains the edges of the graph, each connecting two different nodes or one node with itself.
CREATE TABLE edge_item (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	model INTEGER NOT NULL,
	mutant_connection INTEGER NOT NULL,
	shape VARCHAR(255),
	source INTEGER,
	target INTEGER,
	source_edge INTEGER,
	target_edge INTEGER,
	type INTEGER DEFAULT 0,
	shape_data LONGVARBINARY,
	size DOUBLE DEFAULT 1.0,
	fill_color INTEGER,
	stroke_color INTEGER,
	stroke_width DOUBLE,
	labeled BOOLEAN DEFAULT TRUE,
	visible BOOLEAN DEFAULT TRUE,
	data_group VARCHAR(64),
	CONSTRAINT FK_EDGE_MODEL FOREIGN KEY(model) REFERENCES model(id) ON DELETE CASCADE,
	CONSTRAINT FK_EDGE_MUTANT_CONNECTION FOREIGN KEY(mutant_connection) REFERENCES mutant_connection(id) ON DELETE CASCADE,
	CONSTRAINT FK_EDGE_GRAPHIC FOREIGN KEY(shape) REFERENCES graphic(id),
	CONSTRAINT FK_EDGE_SOURCE_NODE FOREIGN KEY(source) REFERENCES node_item(id) ON DELETE CASCADE,
	CONSTRAINT FK_EDGE_TARGET_NODE FOREIGN KEY(target) REFERENCES node_item(id) ON DELETE CASCADE,
	CONSTRAINT FK_EDGE_SOURCE_EDGE FOREIGN KEY(source_edge) REFERENCES edge_item(id) ON DELETE CASCADE,
	CONSTRAINT FK_EDGE_TARGET_EDGE FOREIGN KEY(target_edge) REFERENCES edge_item(id) ON DELETE CASCADE,
	PRIMARY KEY(id)
);

-- Labels belonging to a node or edge.
CREATE TABLE label_item (
	id INTEGER GENERATED BY DEFAULT AS IDENTITY(START WITH 1) NOT NULL,
	node INTEGER,
	edge INTEGER,
	text LONGVARCHAR,
	x DOUBLE,
	y DOUBLE,
	font VARCHAR(255),
	font_color INTEGER,
	font_size INTEGER DEFAULT 10,
	visible BOOLEAN DEFAULT TRUE,
	h_align CHAR(1) DEFAULT 2,
	v_align CHAR(1) DEFAULT 2,
	CONSTRAINT FK_LABEL_NODE FOREIGN KEY(node) REFERENCES node_item(id) ON DELETE CASCADE,
	CONSTRAINT FK_LABEL_EDGE FOREIGN KEY(edge) REFERENCES edge_item(id) ON DELETE CASCADE,
	PRIMARY KEY(id) 
);

CREATE TABLE attribute_visibility (
	attribute INTEGER NOT NULL,
	node INTEGER,
	edge INTEGER,
	CONSTRAINT FK_ATTRIBUTE_VISIBILITY_ATTRIBUTE FOREIGN KEY(attribute) REFERENCES attribute(id) ON DELETE CASCADE,
	CONSTRAINT FK_ATTRIBUTE_VISIBILITY_NODE FOREIGN KEY(node) REFERENCES node_item(id) ON DELETE CASCADE,
	CONSTRAINT FK_ATTRIBUTE_VISIBILITY_EDGE FOREIGN KEY(edge) REFERENCES edge_item(id) ON DELETE CASCADE,
	UNIQUE(attribute, node, edge)
);

CREATE TABLE property (
	name VARCHAR(255) NOT NULL,
	value VARCHAR(255),
	PRIMARY KEY (name)
);